@@ -1,17 +1,23 @@
 /*
     SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2020 David Edmundson <davidedmundson@kde.org>
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
+#include "abstract_data_source.h"
 #include "seat_interface.h"
 #include "seat_interface_p.h"
 #include "display.h"
 #include "datadevice_interface.h"
 #include "datasource_interface.h"
+#include "datacontroldevice_v1_interface.h"
+#include "datacontrolsource_v1_interface.h"
 #include "keyboard_interface.h"
 #include "keyboard_interface_p.h"
 #include "pointer_interface.h"
 #include "pointer_interface_p.h"
+#include "primaryselectiondevice_v1_interface.h"
+#include "primaryselectionsource_v1_interface.h"
 #include "surface_interface.h"
 #include "textinput_interface_p.h"
 // Qt
@@ -253,26 +259,17 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
     auto dataDeviceCleanup = [this, dataDevice] {
         dataDevices.removeOne(dataDevice);
         keys.focus.selections.removeOne(dataDevice);
-
-        if (currentSelection == dataDevice) {
-            // current selection is cleared
-            currentSelection = nullptr;
-            emit q->selectionChanged(nullptr);
-            for (auto selection: keys.focus.selections) {
-                selection->sendClearSelection();
-            }
-        }
     };
     QObject::connect(dataDevice, &QObject::destroyed, q, dataDeviceCleanup);
     QObject::connect(dataDevice, &Resource::unbound, q, dataDeviceCleanup);
     QObject::connect(dataDevice, &DataDeviceInterface::selectionChanged, q,
         [this, dataDevice] {
-            updateSelection(dataDevice, true);
+            updateSelection(dataDevice);
         }
     );
     QObject::connect(dataDevice, &DataDeviceInterface::selectionCleared, q,
         [this, dataDevice] {
-            updateSelection(dataDevice, false);
+            updateSelection(dataDevice);
         }
     );
     QObject::connect(dataDevice, &DataDeviceInterface::dragStarted, q,
@@ -331,13 +328,78 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
         // same client?
         if (keys.focus.surface->client() == dataDevice->client()) {
             keys.focus.selections.append(dataDevice);
-            if (currentSelection && currentSelection->selection()) {
+            if (currentSelection) {
                 dataDevice->sendSelection(currentSelection);
             }
         }
     }
 }
 
+void SeatInterface::Private::registerDataControlDevice(DataControlDeviceV1Interface *dataDevice)
+{
+    Q_ASSERT(dataDevice->seat() == q);
+    dataControlDevices << dataDevice;
+    auto dataDeviceCleanup = [this, dataDevice] {
+        dataControlDevices.removeOne(dataDevice);
+    };
+    QObject::connect(dataDevice, &QObject::destroyed, q, dataDeviceCleanup);
+
+    QObject::connect(dataDevice, &DataControlDeviceV1Interface::selectionChanged, q,
+                     [this, dataDevice] {
+        // Special klipper workaround to avoid a race
+        // If the mimetype x-kde-onlyReplaceEmpty is set, and we've had another update in the meantime, do nothing
+        // See https://github.com/swaywm/wlr-protocols/issues/92
+        if  (dataDevice->selection() && dataDevice->selection()->mimeTypes().contains(QLatin1String("application/x-kde-onlyReplaceEmpty")) &&
+             currentSelection) {
+            dataDevice->selection()->cancel();
+            return;
+        }
+        q->setSelection(dataDevice->selection());
+    }
+    );
+
+    QObject::connect(dataDevice, &DataControlDeviceV1Interface::selectionCleared, q,
+        [this, dataDevice] {
+            Q_UNUSED(dataDevice);
+            q->setSelection(nullptr);
+        }
+    );
+    if (currentSelection) {
+        dataDevice->sendSelection(currentSelection);
+    }
+}
+
+void SeatInterface::Private::registerPrimarySelectionDevice(PrimarySelectionDeviceV1Interface *primarySelectionDevice)
+{
+    Q_ASSERT(primarySelectionDevice->seat() == q);
+
+    primarySelectionDevices << primarySelectionDevice;
+    auto dataDeviceCleanup = [this, primarySelectionDevice] {
+        primarySelectionDevices.removeOne(primarySelectionDevice);
+        keys.focus.primarySelections.removeOne(primarySelectionDevice);
+    };
+    QObject::connect(primarySelectionDevice, &QObject::destroyed, q, dataDeviceCleanup);
+    QObject::connect(primarySelectionDevice, &PrimarySelectionDeviceV1Interface::selectionChanged, q,
+        [this, primarySelectionDevice] {
+            updatePrimarySelection(primarySelectionDevice);
+        }
+    );
+    QObject::connect(primarySelectionDevice, &PrimarySelectionDeviceV1Interface::selectionCleared, q,
+        [this, primarySelectionDevice] {
+            updatePrimarySelection(primarySelectionDevice);
+        }
+    );
+    // is the new DataDevice for the current keyoard focus?
+    if (keys.focus.surface) {
+        // same client?
+        if (*keys.focus.surface->client() == primarySelectionDevice->client()) {
+            keys.focus.primarySelections.append(primarySelectionDevice);
+            if (currentPrimarySelection) {
+                primarySelectionDevice->sendSelection(currentPrimarySelection);
+            }
+        }
+    }
+}
 
 void SeatInterface::Private::registerTextInput(TextInputInterface *ti)
 {
@@ -382,44 +444,22 @@ void SeatInterface::Private::endDrag(quint32 serial)
     emit q->dragEnded();
 }
 
-void SeatInterface::Private::cancelPreviousSelection(DataDeviceInterface *dataDevice)
+void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice)
 {
-    if (!currentSelection) {
+    // if the update is from the focussed window we should inform the active client
+    if (!(keys.focus.surface && (keys.focus.surface->client() == dataDevice->client()))) {
         return;
     }
-    if (auto s = currentSelection->selection()) {
-        if (currentSelection != dataDevice) {
-            // only if current selection is not on the same device
-            // that would cancel the newly set source
-            s->cancel();
-        }
-    }
+    q->setSelection(dataDevice->selection());
 }
 
-void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice, bool set)
+void SeatInterface::Private::updatePrimarySelection(PrimarySelectionDeviceV1Interface *primarySelectionDevice)
 {
-    bool selChanged = currentSelection != dataDevice;
-    if (keys.focus.surface && (keys.focus.surface->client() == dataDevice->client())) {
-        // cancel the previous selection
-        cancelPreviousSelection(dataDevice);
-        // new selection on a data device belonging to current keyboard focus
-        currentSelection = dataDevice;
-    }
-    if (dataDevice == currentSelection) {
-        // need to send out the selection
-        for (auto focussedDevice: qAsConst(keys.focus.selections)) {
-            if (set) {
-                focussedDevice->sendSelection(dataDevice);
-            } else {
-                focussedDevice->sendClearSelection();
-                currentSelection = nullptr;
-                selChanged = true;
-            }
-        }
-    }
-    if (selChanged) {
-        emit q->selectionChanged(currentSelection);
+    // if the update is from the focussed window we should inform the active client
+    if (!(keys.focus.surface && (*keys.focus.surface->client() == primarySelectionDevice->client()))) {
+        return;
     }
+    q->setPrimarySelection(primarySelectionDevice->selection());
 }
 
 void SeatInterface::setHasKeyboard(bool has)
@@ -648,6 +688,7 @@ void SeatInterface::setDragTarget(SurfaceInterface *surface, const QPointF &glob
         d->drag.target->updateDragTarget(nullptr, serial);
     }
 
+
     // TODO: technically we can have mulitple data devices
     // and we should send the drag to all of them, but that seems overly complicated
     // in practice so far the only case for mulitple data devices is for clipboard overriding
@@ -1144,6 +1185,22 @@ void SeatInterface::setFocusedKeyboardSurface(SurfaceInterface *surface)
                 dataDevice->sendClearSelection();
             }
         }
+        // primary selection
+        QVector<PrimarySelectionDeviceV1Interface *> primarySelectionDevices;
+        for (auto it = d->primarySelectionDevices.constBegin(); it != d->primarySelectionDevices.constEnd(); ++it) {
+            if ((*it)->client() == *surface->client()) {
+                primarySelectionDevices << *it;
+            }
+        }
+
+        d->keys.focus.primarySelections = primarySelectionDevices;
+        for (auto primaryDataDevice : primarySelectionDevices) {
+            if (d->currentSelection) {
+                primaryDataDevice->sendSelection(d->currentPrimarySelection);
+            } else {
+                primaryDataDevice->sendClearSelection();
+            }
+        }
     }
     for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
         (*it)->setFocusedSurface(surface, serial);
@@ -1567,7 +1624,7 @@ void SeatInterface::setFocusedTextInputSurface(SurfaceInterface *surface)
     }
     d->textInput.focus.textInput = t;
     if (d->textInput.focus.surface) {
-        d->textInput.focus.destroyConnection = connect(surface, &Resource::aboutToBeUnbound, this,
+        d->textInput.focus.destroyConnection = connect(surface, &SurfaceInterface::aboutToBeDestroyed, this,
             [this] {
                 setFocusedTextInputSurface(nullptr);
             }
@@ -1595,29 +1652,82 @@ TextInputInterface *SeatInterface::focusedTextInput() const
     return d->textInput.focus.textInput;
 }
 
-DataDeviceInterface *SeatInterface::selection() const
+AbstractDataSource *SeatInterface::selection() const
 {
     Q_D();
     return d->currentSelection;
 }
 
-void SeatInterface::setSelection(DataDeviceInterface *dataDevice)
+void SeatInterface::setSelection(AbstractDataSource *selection)
 {
     Q_D();
-    if (d->currentSelection == dataDevice) {
+    if (d->currentSelection == selection) {
         return;
     }
-    // cancel the previous selection
-    d->cancelPreviousSelection(dataDevice);
-    d->currentSelection = dataDevice;
-    for (auto focussedDevice: qAsConst(d->keys.focus.selections)) {
-        if (dataDevice && dataDevice->selection()) {
-            focussedDevice->sendSelection(dataDevice);
+
+    if (d->currentSelection) {
+        d->currentSelection->cancel();
+        disconnect(d->currentSelection, nullptr, this, nullptr);
+    }
+
+    if (selection) {
+        auto cleanup = [this]() {
+            setSelection(nullptr);
+        };
+        connect(selection, &DataSourceInterface::unbound, this, cleanup);
+        connect(selection, &QObject::destroyed, this, cleanup);
+    }
+
+    d->currentSelection = selection;
+
+    for (auto focussedSelection: qAsConst(d->keys.focus.selections)) {
+        if (selection) {
+            focussedSelection->sendSelection(selection);
         } else {
-            focussedDevice->sendClearSelection();
+            focussedSelection->sendClearSelection();
         }
     }
-    emit selectionChanged(dataDevice);
+
+    for (auto control : qAsConst(d->dataControlDevices)) {
+        if (selection) {
+            control->sendSelection(selection);
+        } else {
+            control->sendClearSelection();
+        }
+    }
+
+    emit selectionChanged(selection);
+}
+
+void SeatInterface::setPrimarySelection(AbstractDataSource *selection)
+{
+    Q_D();
+    if (d->currentPrimarySelection == selection) {
+        return;
+    }
+    if (d->currentPrimarySelection) {
+        d->currentPrimarySelection->cancel();
+        disconnect(d->currentPrimarySelection, nullptr, this, nullptr);
+    }
+
+    if (selection) {
+        auto cleanup = [this]() {
+            setPrimarySelection(nullptr);
+        };
+        connect(selection, &DataSourceInterface::unbound, this, cleanup);
+    }
+
+    d->currentPrimarySelection = selection;
+
+    for (auto focussedSelection: qAsConst(d->keys.focus.primarySelections)) {
+        if (selection) {
+            focussedSelection->sendSelection(selection);
+        } else {
+            focussedSelection->sendClearSelection();
+        }
+    }
+
+    emit primarySelectionChanged(selection);
 }
 
 }
