@@ -1,14 +1,15 @@
 /*
     SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2020 Vlad Zahorodnii <vlad.zahorodnii@kde.org>
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 #ifndef WAYLAND_SERVER_SURFACE_INTERFACE_H
 #define WAYLAND_SERVER_SURFACE_INTERFACE_H
 
-#include "resource.h"
 #include "output_interface.h"
 
+#include <QMatrix4x4>
 #include <QObject>
 #include <QPointer>
 #include <QRegion>
@@ -17,20 +18,16 @@
 
 namespace KWaylandServer
 {
-class BlurManagerInterface;
 class BlurInterface;
 class BufferInterface;
 class ConfinedPointerInterface;
 class ContrastInterface;
-class ContrastManagerInterface;
 class CompositorInterface;
-class IdleInhibitManagerUnstableV1Interface;
 class LockedPointerInterface;
-class PointerConstraintsUnstableV1Interface;
-class ShadowManagerInterface;
 class ShadowInterface;
 class SlideInterface;
 class SubSurfaceInterface;
+class SurfaceInterfacePrivate;
 
 /**
  * @brief Resource representing a wl_surface.
@@ -57,7 +54,7 @@ class SubSurfaceInterface;
  * @see ShadowInterface
  * @see SlideInterface
  **/
-class KWAYLANDSERVER_EXPORT SurfaceInterface : public Resource
+class KWAYLANDSERVER_EXPORT SurfaceInterface : public QObject
 {
     Q_OBJECT
     /**
@@ -72,43 +69,121 @@ class KWAYLANDSERVER_EXPORT SurfaceInterface : public Resource
      * The current input region.
      **/
     Q_PROPERTY(QRegion input READ input NOTIFY inputChanged)
-    Q_PROPERTY(qint32 scale READ scale NOTIFY scaleChanged)
-    Q_PROPERTY(KWaylandServer::OutputInterface::Transform transform READ transform NOTIFY transformChanged)
+    Q_PROPERTY(qint32 bufferScale READ bufferScale NOTIFY bufferScaleChanged)
+    Q_PROPERTY(KWaylandServer::OutputInterface::Transform bufferTransform READ bufferTransform NOTIFY bufferTransformChanged)
     Q_PROPERTY(QSize size READ size NOTIFY sizeChanged)
 public:
-    virtual ~SurfaceInterface();
+    explicit SurfaceInterface(CompositorInterface *compositor, wl_resource *resource);
+    ~SurfaceInterface() override;
+
+    /**
+     * Returns the object id for this Wayland surface.
+     */
+    uint32_t id() const;
+    /**
+     * Returns the Wayland client that owns this SurfaceInterface.
+     */
+    ClientConnection *client() const;
+    /**
+     * Returns the Wayland resource corresponding to this SurfaceInterface.
+     */
+    wl_resource *resource() const;
+    /**
+     * Returns the compositor for this SurfaceInterface.
+     */
+    CompositorInterface *compositor() const;
+
+    /**
+     * Returns a list with all created Wayland surfaces.
+     */
+    static QList<SurfaceInterface *> surfaces();
+
+    /**
+     * Maps the specified @a point from the surface-local coordinates to buffer pixel coordinates.
+     *
+     * Note that there is no direct connection between points in the surface-local coordinates
+     * and points in the buffer pixel coordinates. In order to map points between the two spaces,
+     * one has to use mapToBuffer() and mapFromBuffer().
+     *
+     * The returned value will become invalid when the surfaceToBufferMatrixChanged() signal is emitted.
+     *
+     * @see surfaceToBufferMatrix(), surfaceToBufferMatrixChanged()
+     * @since 5.20
+     */
+    QPointF mapToBuffer(const QPointF &point) const;
+    /**
+     * Maps the specified @a point from the buffer pixel coordinates to surface-local coordinates.
+     *
+     * Note that there is no direct connection between points in the surface-local coordinates
+     * and points in the buffer pixel coordinates. In order to map points between the two spaces,
+     * one has to use mapToBuffer() and mapFromBuffer().
+     *
+     * The returned value will become invalid when the surfaceToBufferMatrixChanged() signal is emitted.
+     *
+     * @see surfaceToBufferMatrix(), surfaceToBufferMatrixChanged()
+     * @since 5.20
+     */
+    QPointF mapFromBuffer(const QPointF &point) const;
+    /**
+     * Maps the specified @a region from the surface-local coordinates to buffer pixel coordinates.
+     *
+     * Note that there is no direct connection between regions in the surface-local coordinates
+     * and regions in the buffer pixel coordinates. In order to map regions between the two spaces,
+     * one has to use mapToBuffer() and mapFromBuffer().
+     *
+     * The returned value will become invalid when the surfaceToBufferMatrixChanged() signal is emitted.
+     *
+     * @see surfaceToBufferMatrix(), surfaceToBufferMatrixChanged()
+     * @since 5.20
+     */
+    QRegion mapToBuffer(const QRegion &region) const;
+    /**
+     * Maps the specified @a region from the buffer pixel coordinates to surface-local coordinates.
+     *
+     * Note that there is no direct connection between regions in the surface-local coordinates
+     * and regions in the buffer pixel coordinates. In order to map regions between the two spaces,
+     * one has to use mapToBuffer() and mapFromBuffer().
+     *
+     * The returned value will become invalid when the surfaceToBufferMatrixChanged() signal is emitted.
+     *
+     * @see surfaceToBufferMatrix(), surfaceToBufferMatrixChanged()
+     * @since 5.20
+     */
+    QRegion mapFromBuffer(const QRegion &region) const;
+    /**
+     * Returns the projection matrix from the surface-local coordinates to buffer coordinates.
+     *
+     * @see surfaceToBufferMatrixChanged()
+     * @since 5.20
+     */
+    QMatrix4x4 surfaceToBufferMatrix() const;
 
     void frameRendered(quint32 msec);
 
     QRegion damage() const;
     QRegion opaque() const;
     QRegion input() const;
-#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 5)
-    /**
-     * Use Surface::inputIsInfinite instead.
-     * @deprecated Since 5.5, use inputIsInfinite
-     */
-    KWAYLANDSERVER_DEPRECATED_VERSION(5, 5, "Use SurfaceInterface::inputIsInfinite()")
-    bool inputIsInfitine() const;
-#endif
-    /**
-     * Replaces Surface::inputIsInfitine instead.
-     * @since 5.5
-     */
     bool inputIsInfinite() const;
-    qint32 scale() const;
-    OutputInterface::Transform transform() const;
+    qint32 bufferScale() const;
+    /**
+     * Returns the buffer transform that had been applied to the buffer to compensate for
+     * output rotation.
+     *
+     * If the surface is on an output that is rotated 90 degrees clockwise, the buffer will
+     * be rotated 90 degrees counter clockwise.
+     */
+    OutputInterface::Transform bufferTransform() const;
     /**
      * @returns the current BufferInterface, might be @c nullptr.
      **/
     BufferInterface *buffer();
     QPoint offset() const;
     /**
-     * The size of the Surface in global compositor space.
-     * @see For buffer size use BufferInterface::size
-     * from SurfaceInterface::buffer
-     * @since 5.3
-     **/
+     * Returns the current size of the surface, in surface coordinates.
+     *
+     * Note that there is no direct relationship between the surface size and the buffer size.
+     * In order to determine the size of the currently attached buffer, use buffer()->size().
+     */
     QSize size() const;
     /**
      * Returns the rectangle that bounds this surface and all of its sub-surfaces.
@@ -118,6 +193,12 @@ public:
      * @since 5.69
      */
     QRect boundingRect() const;
+    /**
+     * Returns the size of the attached buffer, in device pixels.
+     *
+     * If no buffer is attached to this surface, an invalid QSize will be returned.
+     */
+    QSize bufferSize() const;
 
     /**
      * @returns The SubSurface for this Surface in case there is one.
@@ -286,6 +367,24 @@ public:
     SurfaceInterface* dataProxy() const;
 
 Q_SIGNALS:
+    /**
+     * This signal is emitted when the underlying wl_surface resource is about to be freed.
+     *
+     * The unbound() signal is emitted either when the client that owns the surface has been
+     * destroyed or if the surface has been destroyed due to a destructor request.
+     *
+     * The SurfaceInterface object and the associated wl_surface resource are valid when this
+     * signal is emitted.
+     */
+    void aboutToBeDestroyed();
+    /**
+     * This signal is emitted when the projection matrix from the surface-local coordinate space
+     * to the buffer coordinate space has been changed.
+     *
+     * Note that the compositor will most likely need to re-compute the texture coordinates after
+     * the surface-to-buffer matrix has been changed.
+     */
+    void surfaceToBufferMatrixChanged();
     /**
      * Emitted whenever the SurfaceInterface got damaged.
      * The signal is only emitted during the commit of state.
@@ -297,8 +396,18 @@ Q_SIGNALS:
     void damaged(const QRegion&);
     void opaqueChanged(const QRegion&);
     void inputChanged(const QRegion&);
-    void scaleChanged(qint32);
-    void transformChanged(KWaylandServer::OutputInterface::Transform);
+    /**
+     * This signal is emitted when the scale of the attached buffer has changed.
+     */
+    void bufferScaleChanged(qint32);
+    /**
+     * This signal is emitted when the buffer transform has changed.
+     */
+    void bufferTransformChanged(KWaylandServer::OutputInterface::Transform);
+    /**
+     * This signal is emitted when the size of the attached buffer has changed.
+     */
+    void bufferSizeChanged();
     /**
      * Emitted when the Surface becomes visible, i.e. a non-null buffer has been attached.
      **/
@@ -308,8 +417,8 @@ Q_SIGNALS:
      **/
     void unmapped();
     /**
-     * @since 5.3
-     **/
+     * This signal is emitted when the surface size has changed.
+     */
     void sizeChanged();
     /**
      * @since 5.4
@@ -372,19 +481,8 @@ Q_SIGNALS:
     void committed();
 
 private:
-    friend class CompositorInterface;
-    friend class SubSurfaceInterface;
-    friend class ShadowManagerInterface;
-    friend class BlurManagerInterface;
-    friend class SlideManagerInterface;
-    friend class ContrastManagerInterface;
-    friend class IdleInhibitManagerUnstableV1Interface;
-    friend class PointerConstraintsUnstableV1Interface;
-    friend class SurfaceRole;
-    explicit SurfaceInterface(CompositorInterface *parent, wl_resource *parentResource);
-
-    class Private;
-    Private *d_func() const;
+    QScopedPointer<SurfaceInterfacePrivate> d;
+    friend class SurfaceInterfacePrivate;
 };
 
 }
